@@ -2,12 +2,12 @@ module Roo
   class Excelx
     class Cell
       class String < Cell::Base
-        attr_reader :value, :formula, :format, :cell_type, :cell_value, :link, :coordinate
+        attr_reader :value, :formula, :format, :cell_value, :coordinate
+
+        attr_reader_with_default default_type: :string, cell_type: :string
 
         def initialize(value, formula, style, link, coordinate)
           super(value, formula, nil, style, link, coordinate)
-          @type = @cell_type = :string
-          @value = link? ? Roo::Link.new(link, value) : value
         end
 
         def empty?
